package com.weebkun.speedq_printer;

import android.bluetooth.BluetoothAdapter;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.util.Log;

import net.posprinter.posprinterface.IMyBinder;
import net.posprinter.posprinterface.UiExecute;
import net.posprinter.service.PosprinterService;
import net.posprinter.utils.DataForSendToPrinterPos58;
import net.posprinter.utils.DataForSendToPrinterPos80;
import net.posprinter.utils.PosPrinterDev;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

/**
 * class to manage bluetooth discovery, connections, and devices
 */
public class PrinterManager {
    private static final BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
    private static IMyBinder binder;

    public interface ConnectionStatusListener{
        void onSuccess();
        void onFailure();
    }

    private static final ServiceConnection conn = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
            System.out.println("bound");
            //Bind successfully
            binder = (IMyBinder) iBinder;
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {
        }
    };

    public static Device[] getPairedDevices() {
        Device[] devices = adapter.getBondedDevices().stream().map(Device::new).toArray(Device[]::new);
        for(Device device : devices) {
            System.out.println(device.name);
        }
        return devices;
    }

    public static void connectBT(String mac, final ConnectionStatusListener listener) {
        System.out.println("connect");
        binder.connectBtPort(mac, new UiExecute() {
            @Override
            public void onsucess() {
                Log.i("bt", "connected to printer");
                listener.onSuccess();
                binder.write(DataForSendToPrinterPos80.openOrCloseAutoReturnPrintState(0x1f), new UiExecute() {
                    @Override
                    public void onsucess() {
                        binder.acceptdatafromprinter(new UiExecute() {
                            @Override
                            public void onsucess() {
                            }

                            @Override
                            public void onfailed() {
                            }
                        });
                    }

                    @Override
                    public void onfailed() {
                        listener.onFailure();
                    }
                });
            }

            @Override
            public void onfailed() {
                Log.e("bt", "cannot connect");
            }
        });
    }

    public static void connectUSB(Context context, ConnectionStatusListener listener) {
        String usbAddress = PosPrinterDev.GetUsbPathNames(context).get(0);
        binder.connectUsbPort(context, usbAddress, new UiExecute() {
            @Override
            public void onsucess() {
                listener.onSuccess();
                System.out.println("usb connect success");
            }

            @Override
            public void onfailed() {
                listener.onFailure();
                System.out.println("usb connect failure");
            }
        });
    }

    public static void bindService(Context context) {
        System.out.println("bind");
        context.bindService(new Intent(context, PosprinterService.class), conn, Context.BIND_AUTO_CREATE);
    }

    public static void printText(String text){

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    List<byte[]> list = new ArrayList<>();
                    //initialize the printer
//                            list.add( DataForSendToPrinterPos58.initializePrinter());
                    list.add(DataForSendToPrinterPos58.initializePrinter());
                    byte[] data1 = stringToBytes(text);
                    list.add(data1);
                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static byte[] stringToBytes(String str){
        byte[] data = null;
        try {
            data = str.getBytes("gbk");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return data;
    }

    public static void printNewLine(){
        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    List<byte[]> list = new ArrayList<>();
                    //initialize the printer
//                            list.add( DataForSendToPrinterPos58.initializePrinter());
                    list.add(DataForSendToPrinterPos58.initializePrinter());
                    list.add(PrinterCommands.FEED_LINE);
                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeed(1));
                    return list;
                });
    }

    public static byte[] getSize(int size){

        // Print config "mode"
        byte[] cc = new byte[] { 0x1B, 0x21, 0x03 }; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[] { 0x1B, 0x21, 0x08 }; // 1- only bold text
        byte[] bb2 = new byte[] { 0x1B, 0x21, 0x20 }; // 2- bold with medium text
        byte[] bb3 = new byte[] { 0x1B, 0x21, 0x10 }; // 3- bold with large text
        byte[] bb4 = new byte[] { 0x1B, 0x21, 0x30 }; // 4- strong text

        switch (size) {
            case 0:
                return bb;
            case 1:
                return bb2;
            case 2:
                return bb3;
            case 3:
                return bb4;
            default:
                return cc;
        }
    }

    public static void printOrder(String message){
        System.out.println("printCenter");
        List<byte[]> list = new ArrayList<>();

        list.add(DataForSendToPrinterPos58.initializePrinter());

        // Set Larger font size for order
        list.add(DataForSendToPrinterPos58.selectCharacterSize(0x33));

        // Center align text
        list.add(PrinterCommands.ESC_ALIGN_CENTER);
        // Add text
        list.add(message.getBytes());

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void printCenter(String message, int size){
        System.out.println("printCenter");

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());
        // Size of text
        list.add(getSize(size));
        // Center align text
        list.add(PrinterCommands.ESC_ALIGN_CENTER);
        // Add text
        list.add(message.getBytes());

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void printLeft(String message, int size){
        System.out.println("printLeft");

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());

        // Size of text
        list.add(getSize(size));
        // Left align text
        list.add(PrinterCommands.ESC_ALIGN_LEFT);
        // Add text
        list.add(message.getBytes());

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void printRight(String message, int size){
        System.out.println("printRight");

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());

        // Size of text
        list.add(getSize(size));
        // Right align text
        list.add(PrinterCommands.ESC_ALIGN_RIGHT);
        // Add text
        list.add(message.getBytes());

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void print3Column(String msg1, String msg2, String msg3, int size) {
        System.out.println("print3Column");

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());
        list.add(DataForSendToPrinterPos58.selectCharacterSize(0x01));

        // Size of text
        //list.add(getSize(size));
        // Center align text
        list.add(PrinterCommands.ESC_ALIGN_CENTER);
        // Format the text
        String line = String.format("%-4s %-20s %5s", msg1, msg2, msg3);
        // Add text
        list.add(line.getBytes());

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }
}
