package com.weebkun.speedq_printer;

import android.content.Context;

import androidx.annotation.NonNull;

import com.squareup.moshi.Moshi;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** SpeedqPrinterPlugin */
public class SpeedqPrinterPlugin implements FlutterPlugin, MethodCallHandler{
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private Context context;
    private FlutterPluginBinding pluginBinding;
    public final Moshi moshi = new Moshi.Builder().build();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        pluginBinding = flutterPluginBinding;
        channel = new MethodChannel(pluginBinding.getBinaryMessenger(), "speedq_printer");
        channel.setMethodCallHandler(this);
        context = pluginBinding.getApplicationContext();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {

        final Map<String, Object> arguments = call.arguments();

        switch(call.method) {
            case "getPlatformVersion":
                result.success("Android " + android.os.Build.VERSION.RELEASE);
                break;
            case "init":
                PrinterManager.bindService(context);
                result.success(null);
            case "getPairedDevices":
                result.success(moshi.adapter(Device[].class).toJson(PrinterManager.getPairedDevices()));
                break;
            case "connectBT":
                PrinterManager.bindService(context);
                PrinterManager.connectBT((String) arguments.get("mac"), new PrinterManager.ConnectionStatusListener() {
                    @Override
                    public void onSuccess() {
                        result.success(true);
                    }

                    @Override
                    public void onFailure() {
                        result.success(false);
                    }
                });
                break;
            case "connectUSB":
                PrinterManager.connectUSB(context, new PrinterManager.ConnectionStatusListener() {
                    @Override
                    public void onSuccess() {
                        result.success(true);
                    }

                    @Override
                    public void onFailure() {
                        result.success(false);
                    }
                });
            case "print":
                PrinterManager.printText(((String) arguments.get("message")));
                result.success(null);
                break;
            case "printNewLine":
                PrinterManager.printNewLine();
                result.success(null);
                break;
            case "printOrder":
                if (arguments.containsKey("message")){
                    String message = (String) arguments.get("message");
                PrinterManager.printOrder(message);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "printCenter":
                if (arguments.containsKey("message")) {
                    String message = (String) arguments.get("message");
                    int size = (int) arguments.get("size");
                    PrinterManager.printCenter(message, size);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "printLeft":
                if (arguments.containsKey("message")) {
                    String message = (String) arguments.get("message");
                    int size = (int) arguments.get("size");
                    PrinterManager.printLeft(message, size);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "printRight":
                if (arguments.containsKey("message")) {
                    String message = (String) arguments.get("message");
                    int size = (int) arguments.get("size");
                    PrinterManager.printRight(message, size);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
            case "print3Column":
                if (arguments.containsKey("string1")) {
                    String string1 = (String) arguments.get("string1");
                    String string2 = (String) arguments.get("string2");
                    String string3 = (String) arguments.get("string3");
                    int size = (int) arguments.get("size");
                    PrinterManager.print3Column(string1, string2,string3, size);
                } else {
                    result.error("invalid_argument", "argument 'message' not found", null);
                }
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }
}
